// ITDB02_Touch_ButtonTest (C)2010 Henning Karlsen
// web: http://www.henningkarlsen.com/electronics
//
// This program is a quick demo of how create and use buttons.
//
// This program requires the ITDB02_Graph library (8bit mode)
// or ITDB02_Graph16 (16bit mode).
//
// It is assumed that the ITDB02 module is connected to a
// ITDB02 Shield, a ITDB02 Mega Shield or that you know how 
// to change the pin numbers in the setup.

// Remember to change the next line if you are using as 16bit module!
#include "ITDB02_Graph_ChipKIT.h"
#include "ITDB02_Touch.h"


#define xoff 15
#define yoff 10
#define bsize 75
#define txoff 30
// Declare which fonts we will be using
extern uint8_t BigFont[];

//button names
char btnames[]="MTC+PWV-OKV+TXC-MNINMOEX";
char bbuf[3]="";
// Uncomment the next two lines for the ITDB02 Shield
ITDB02        myGLCD(GRS,GWR,GCS,GRST);
ITDB02_Touch  myTouch(15,10,14,9,8);

// Uncomment the next two lines for the ITDB02 Mega Shield
//ITDB02        myGLCD(38,39,40,41);   // Remember to add ASPECT_16x9 if you are using an ITDB02-3.2WC!
//ITDB02_Touch  myTouch(6,5,4,3,2);

int x, y;
char stCurrent[20]="";
int stCurrentLen=0;
char stLast[20]="";

void setup()
{
// Initial setup
  myGLCD.InitLCD();
  myGLCD.clrScr();

  myTouch.InitTouch();
  myTouch.setPrecision(PREC_MEDIUM);

  myGLCD.setFont(BigFont);
  myGLCD.setBackColor(0, 0, 255);
  
for (int j=0; j<4; j++)
{
    for (x=0; x<3; x++)
  {
    myGLCD.setColor(0, 0, 255);
    myGLCD.fillRoundRect (xoff+(x*bsize), yoff+j*bsize, bsize+(x*bsize), bsize+j*bsize);
    myGLCD.setColor(255, 255, 255);
    myGLCD.drawRoundRect (xoff+(x*bsize), yoff+j*bsize, bsize+(x*bsize), bsize+j*bsize);
    
bbuf[0]=btnames[2*(x+3*j)];
bbuf[1]=btnames[2*(x+3*j)+1];
    myGLCD.print(bbuf, txoff+(x*bsize), txoff+j*bsize);
  }
}
}  

// Draw a red frame while a button is touched
void waitForIt(int x1, int y1, int x2, int y2)
{
  myGLCD.setColor(255, 0, 0);
  myGLCD.drawRoundRect (x1, y1, x2, y2);
  while (myTouch.dataAvailable())
    myTouch.read();
  myGLCD.setColor(255, 255, 255);
  myGLCD.drawRoundRect (x1, y1, x2, y2);
}

void loop()
{
  while (true)
  {
    if (myTouch.dataAvailable())
    {
      myTouch.read();
      x=myTouch.getX();
      y=myTouch.getY();
      
      if ((y>=yoff) && (y<=bsize))  // Upper row
      {
        if ((x>=xoff) && (x<=bsize))  // Button: 1
        {
          waitForIt(xoff, yoff, bsize, bsize);
         //action 
        }
        if ((x>=(xoff+bsize)) && (x<=2*bsize))  // Button: 2
        {
          waitForIt(xoff+bsize, yoff, 2*bsize, bsize);
          //action
        }
        if ((x>=(xoff+2*bsize)) && (x<=3*bsize))  // Button: 3
        {
          waitForIt(xoff+2*bsize, yoff, 3*bsize, bsize);
          //action
        }

      }

      if ((y>=yoff+bsize) && (y<=2*bsize))  // second row
      {
        if ((x>=xoff) && (x<=bsize))  // Button: 6
        {
          waitForIt(xoff, yoff+bsize, bsize, 2*bsize);
          //action
        }
        if ((x>=(xoff+bsize)) && (x<=2*bsize))  // Button: 7
        {
          waitForIt(xoff+bsize, yoff+bsize, 2*bsize, 2*bsize);
         //action 
        }
        if ((x>=(xoff+2*bsize)) && (x<=3*bsize))  // Button: 8
        {
          waitForIt(xoff+2*bsize, yoff+bsize, 3*bsize, 2*bsize);
          //action 
        }
       
      }

      if ((y>=yoff+2*bsize) && (y<=3*bsize))  // third row
      {
        if ((x>=xoff) && (x<=bsize))  // Button: 6
        {
          waitForIt(xoff, yoff+2*bsize, bsize, 3*bsize);
          //action
        }
        if ((x>=(xoff+bsize)) && (x<=2*bsize))  // Button: 7
        {
          waitForIt(xoff+bsize, yoff+2*bsize, 2*bsize, 3*bsize);
         //action 
        }
        if ((x>=(xoff+2*bsize)) && (x<=3*bsize))  // Button: 8
        {
          waitForIt(xoff+2*bsize, yoff+2*bsize, 3*bsize, 3*bsize);
          //action
        }
       
      }
      if ((y>=yoff+3*bsize) && (y<=4*bsize))  // fourth row
      {
        if ((x>=xoff) && (x<=bsize))  // Button: 6
        {
          waitForIt(xoff, yoff+3*bsize, bsize, 4*bsize);
          //action
        }
        if ((x>=(xoff+bsize)) && (x<=2*bsize))  // Button: 7
        {
          waitForIt(xoff+bsize, yoff+3*bsize, 2*bsize, 4*bsize);
         //action
        }
        if ((x>=(xoff+2*bsize)) && (x<=3*bsize))  // Button: 8
        {
          waitForIt(xoff+2*bsize, yoff+3*bsize, 3*bsize, 4*bsize);
         //action
        }
       
      }
    }
  }
}



